@@ -98,11 +98,11 @@ my $invalid_label = 0;
 func get_data($layout)
 {
     my ($train_sentences, $vocabulary) = tokenize_text(
-        './data/ptb.train.txt', start_label => $start_label,
+        './data/sherlockholmes.train.txt', start_label => $start_label,
         invalid_label => $invalid_label
     );
     my ($validation_sentences) = tokenize_text(
-        './data/ptb.test.txt', vocab => $vocabulary,
+        './data/sherlockholmes.test.txt', vocab => $vocabulary,
         start_label => $start_label, invalid_label => $invalid_label
     );
     my $data_train  = mx->rnn->BucketSentenceIter(
