@@ -21,6 +21,32 @@ use warnings;
 use AI::MXNet::Contrib::Symbol;
 use AI::MXNet::Contrib::NDArray;
 
+=head1 NAME
+
+    AI::MXNet::Contrib - An interface to experimental operators defined in C++ space.
+=cut
+
+=head1 SYNOPSIS
+
+    my $embed;
+    if($sparse_embedding)
+    {
+        my $embed_weight = mx->sym->Variable('embed_weight', stype=>'row_sparse');
+        $embed = mx->sym->contrib->SparseEmbedding(
+            data=>$data, input_dim=>$num_words,
+            weight=>$embed_weight, output_dim=>$num_embed,
+            name=>'embed'
+        );
+    }
+    else
+    {
+        $embed = mx->sym->Embedding(
+            data=>$data, input_dim=>$num_words,
+            output_dim=>$num_embed, name=>'embed'
+        );
+    }
+=cut
+
 sub sym    { 'AI::MXNet::Contrib::Symbol'  }
 sub symbol { 'AI::MXNet::Contrib::Symbol'  }
 sub nd     { 'AI::MXNet::Contrib::NDArray' }
